# Copy accross the driver, tools and firmware
LOCAL_PATH := $(call my-dir)

# Driver module - built beforehand - strip it first
ALL_PREBUILT += $(TARGET_OUT)/lib/modules/unifi_sdio.ko
$(TARGET_OUT)/lib/modules/unifi_sdio.ko : $(LOCAL_PATH)/driver/unifi_sdio_stripped.ko | $(ACP)
	$(transform-prebuilt-to-target)

$(LOCAL_PATH)/driver/unifi_sdio_stripped.ko : $(LOCAL_PATH)/driver/unifi_sdio.ko
	arm-eabi-strip --strip-unneeded $(dir $@)/unifi_sdio.ko -o $@

# Tools, including udev-helper script
ALL_PREBUILT += $(TARGET_OUT)/bin/hotplug
$(TARGET_OUT)/bin/hotplug : $(LOCAL_PATH)/tools/android/hotplug | $(ACP)
	$(transform-prebuilt-to-target)

ALL_PREBUILT += $(TARGET_OUT)/bin/unififw
$(TARGET_OUT)/bin/unififw : $(LOCAL_PATH)/tools/android/unififw | $(ACP)
	$(transform-prebuilt-to-target)

ifneq ($(UNIFI_BUILD_HELPER),true)

# unifi_helper - prebuilt
ALL_PREBUILT += $(TARGET_OUT)/bin/unifi_helper
$(TARGET_OUT)/bin/unifi_helper : $(LOCAL_PATH)/tools/android/unifi_helper | $(ACP)
	$(transform-prebuilt-to-target)

endif

ifeq (exists, $(shell [ -e $(LOCAL_PATH)/../synergy/paths.mk ] && echo exists))
#
# This file defines all the synergy root paths
# FW_ROOT, BT_ROOT, WIFI_ROOT, NUCLEUS_ROOT etc
# $(SYNERGY_ROOT) MUST BE DEFINED
#
SYNERGY_ROOT=$(LOCAL_PATH)/../synergy
include $(SYNERGY_ROOT)/paths.mk
SYNERGY_WIFI_ROOT := $(WIFI_ROOT)
endif
# Assign some defaults if the file wasn't present in the synergy directory 
SYNERGY_WIFI_ROOT ?= synergy/wifi/main

HIP_BASE_DIR := $(SYNERGY_WIFI_ROOT)/src/hip

# unifi_config application
include $(CLEAR_VARS)
LOCAL_SHARED_LIBRARIES := libc libcutils
LOCAL_SRC_FILES := tools/unifi_config_lib.c \
                   tools/unifi_config.c \
                   tools/android/getsubopt.c
LOCAL_C_INCLUDES := $(KERNEL_HEADERS) \
                    $(HIP_BASE_DIR)/common \
                    $(LOCAL_PATH)/driver \
                    $(SYNERGY_WIFI_ROOT)/src/sme/code/lib_sme/common \
                    $(SYNERGY_WIFI_ROOT)/inc \
                    $(FW_ROOT)/bsp/inc \
                    $(FW_ROOT)/bsp/ports/pclin/inc \
                    $(FW_ROOT)/gsp/inc \
                    $(FW_ROOT)/gsp/deprecated/inc
LOCAL_MODULE := unifi_config
LOCAL_MODULE_TAGS := optional

include $(BUILD_EXECUTABLE)

# unifi_manager application
include $(CLEAR_VARS)
LOCAL_SHARED_LIBRARIES := libc libcutils
LOCAL_SRC_FILES := tools/unifi_manager.c \
                   tools/mibquery.c \
                   tools/sme_drv.c
LOCAL_C_INCLUDES := $(KERNEL_HEADERS) \
                    $(HIP_BASE_DIR)/common \
                    $(HIP_BASE_DIR)/inc \
                    $(LOCAL_PATH)/driver \
                    $(SYNERGY_WIFI_ROOT)/src/sme/code/lib_sme/common \
                    $(SYNERGY_WIFI_ROOT)/inc \
                    $(SYNERGY_WIFI_ROOT)/src/inc \
                    $(FW_ROOT)/bsp/inc \
                    $(FW_ROOT)/bsp/ports/pclin/inc \
                    $(FW_ROOT)/gsp/inc \
                    $(FW_ROOT)/gsp/deprecated/inc
LOCAL_MODULE := unifi_manager
LOCAL_MODULE_TAGS := optional

include $(BUILD_EXECUTABLE)

# Firmware - originals go in /etc/firmware/unifi-sdio-0
UNIFI_FW_PATH := unifi-sdio-0

ALL_PREBUILT += $(TARGET_OUT)/etc/firmware/$(UNIFI_FW_PATH)/staonly.xbv
$(TARGET_OUT)/etc/firmware/$(UNIFI_FW_PATH)/staonly.xbv : $(LOCAL_PATH)/../firmware/staonly.xbv | $(ACP)
	$(transform-prebuilt-to-target)

ALL_PREBUILT += $(TARGET_OUT)/etc/firmware/$(UNIFI_FW_PATH)/ap.xbv
$(TARGET_OUT)/etc/firmware/$(UNIFI_FW_PATH)/ap.xbv : $(LOCAL_PATH)/../firmware/ap.xbv | $(ACP)
	$(transform-prebuilt-to-target)

ALL_PREBUILT += $(TARGET_OUT)/etc/firmware/$(UNIFI_FW_PATH)/ufmib.dat
$(TARGET_OUT)/etc/firmware/$(UNIFI_FW_PATH)/ufmib.dat : $(LOCAL_PATH)/../firmware/ufmib.dat | $(ACP)
	$(transform-prebuilt-to-target)

# Create a symlink for unifi plugged into second sdio slot
# /etc/firmware/unifi-sdio-1 -> /etc/firmware/unifi-sdio-0
SYMLINKS := $(addprefix $(TARGET_OUT),/etc/firmware/unifi-sdio-1)
$(SYMLINKS): $(TARGET_OUT)/etc/firmware/$(UNIFI_FW_PATH)/staonly.xbv
	@echo "Symlink: $@ -> $(UNIFI_FW_PATH)"
	@mkdir -p $(dir $@)
	@rm -rf $@
	$(hide) ln -sf $(UNIFI_FW_PATH) $@

ALL_DEFAULT_INSTALLED_MODULES += $(SYMLINKS)
